@@ -35,6 +35,7 @@ extern "C" {
 #include <stdlib.h>
 #include <unistd.h>
 #include <math.h>
+#include <limits.h>
 
 #include <spa/utils/string.h>
 
@@ -47,10 +48,12 @@ typedef void (*pa_free_cb_t)(void *p);
 #define PA_LIKELY(x) (__builtin_expect(!!(x),1))
 #define PA_UNLIKELY(x) (__builtin_expect(!!(x),0))
 #define PA_PRINTF_FUNC(fmt, arg1) __attribute__((format(printf, fmt, arg1)))
+#define PA_UNUSED __attribute__ ((unused))
 #else
 #define PA_LIKELY(x) (x)
 #define PA_UNLIKELY(x) (x)
 #define PA_PRINTF_FUNC(fmt, arg1)
+#define PA_UNUSED
 #endif
 
 #define PA_MIN(a,b)                    \
@@ -96,7 +99,7 @@ typedef enum pa_available {
 	PA_AVAILABLE_YES = 2,
 } pa_available_t;
 
-#define PA_RATE_MAX (48000U*8U)
+#define PA_RATE_MAX (48000U*16U)
 
 typedef enum pa_sample_format {
 	PA_SAMPLE_U8,		/**< Unsigned 8 Bit PCM */
@@ -214,6 +217,7 @@ typedef enum pa_log_level {
 	PA_LOG_NOTICE = 2,    /* Notice messages */
 	PA_LOG_INFO   = 3,    /* Info messages */
 	PA_LOG_DEBUG  = 4,    /* Debug messages */
+	PA_LOG_TRACE = 5,
 	PA_LOG_LEVEL_MAX
 } pa_log_level_t;
 
@@ -245,6 +249,7 @@ static inline PA_PRINTF_FUNC(5, 6) void pa_log_level_meta(enum pa_log_level leve
 #define pa_log_notice(fmt,...)	pa_logl(PA_LOG_NOTICE, fmt, ##__VA_ARGS__)
 #define pa_log_info(fmt,...)	pa_logl(PA_LOG_INFO, fmt, ##__VA_ARGS__)
 #define pa_log_debug(fmt,...)	pa_logl(PA_LOG_DEBUG, fmt, ##__VA_ARGS__)
+#define pa_log_trace(fmt,...)	pa_logl(PA_LOG_TRACE, fmt, ##__VA_ARGS__)
 #define pa_log			pa_log_error
 
 #define pa_assert_se(expr)                                              \
@@ -658,6 +663,8 @@ static inline char *pa_readlink(const char *p) {
         ssize_t n;
 
         c = pa_xmalloc(l);
+        if (c == NULL)
+            return NULL;
 
         if ((n = readlink(p, c, l-1)) < 0) {
             pa_xfree(c);
@@ -670,6 +677,9 @@ static inline char *pa_readlink(const char *p) {
         }
 
         pa_xfree(c);
+
+        if (l >= (size_t)(INT_MAX / 2))
+            return NULL;
         l *= 2;
     }
 #else
@@ -677,6 +687,8 @@ static inline char *pa_readlink(const char *p) {
 #endif
 }
 
+char *get_data_path(const char *data_dir, const char *data_type, const char *fname);
+
 #include <spa/support/i18n.h>
 
 extern struct spa_i18n *acp_i18n;
