@@ -35,8 +35,9 @@
 #include "defs.h"
 #include "rtp.h"
 #include "media-codecs.h"
+#include "iso-io.h"
 
-static struct spa_log_topic log_topic = SPA_LOG_TOPIC(0, "spa.bluez5.source.media");
+SPA_LOG_TOPIC_DEFINE_STATIC(log_topic, "spa.bluez5.source.media");
 #undef SPA_LOG_TOPIC_DEFAULT
 #define SPA_LOG_TOPIC_DEFAULT &log_topic
 
@@ -147,6 +148,8 @@ struct impl {
 	uint8_t buffer_read[4096];
 	struct timespec now;
 	uint64_t sample_count;
+
+	uint32_t errqueue_count;
 };
 
 #define CHECK_PORT(this,d,p)    ((d) == SPA_DIRECTION_OUTPUT && (p) == 0)
@@ -454,6 +457,24 @@ static int32_t decode_data(struct impl *this, uint8_t *src, uint32_t src_size,
 	return dst_size - avail;
 }
 
+static void handle_errqueue(struct impl *this)
+{
+	int res;
+
+	/* iso-io/media-sink use these for TX latency.
+	 * Someone else should be reading them, so drop
+	 * only after yielding.
+	 */
+	if (this->errqueue_count < 4) {
+		this->errqueue_count++;
+		return;
+	}
+
+	this->errqueue_count = 0;
+	res = recv(this->fd, NULL, 0, MSG_ERRQUEUE | MSG_TRUNC);
+	spa_log_trace(this->log, "%p: ignoring errqueue data (%d)", this, res);
+}
+
 static void media_on_ready_read(struct spa_source *source)
 {
 	struct impl *this = source->data;
@@ -466,6 +487,11 @@ static void media_on_ready_read(struct spa_source *source)
 
 	/* make sure the source is an input */
 	if ((source->rmask & SPA_IO_IN) == 0) {
+		if (source->rmask & SPA_IO_ERR) {
+			handle_errqueue(this);
+			return;
+		}
+
 		spa_log_error(this->log, "source is not an input, rmask=%d", source->rmask);
 		goto stop;
 	}
@@ -474,6 +500,8 @@ static void media_on_ready_read(struct spa_source *source)
 		goto stop;
 	}
 
+	this->errqueue_count = 0;
+
 	spa_log_trace(this->log, "socket poll");
 
 	/* read */
@@ -526,6 +554,8 @@ static void media_on_ready_read(struct spa_source *source)
 stop:
 	if (this->source.loop)
 		spa_loop_remove_source(this->data_loop, &this->source);
+	if (this->transport && this->transport->iso_io)
+		spa_bt_iso_io_set_cb(this->transport->iso_io, NULL, NULL);
 }
 
 static int setup_matching(struct impl *this)
@@ -590,7 +620,7 @@ static void media_on_timeout(struct spa_source *source)
 
 	setup_matching(this);
 
-	this->next_time = now_time + duration * SPA_NSEC_PER_SEC / port->buffer.corr / rate;
+	this->next_time = (uint64_t)(now_time + duration * SPA_NSEC_PER_SEC / port->buffer.corr / rate);
 
 	if (SPA_LIKELY(this->clock)) {
 		this->clock->nsec = now_time;
@@ -612,6 +642,21 @@ static void media_on_timeout(struct spa_source *source)
 	set_timeout(this, this->next_time);
 }
 
+static void media_iso_pull(struct spa_bt_iso_io *iso_io)
+{
+	/* TODO: eventually use iso-io here, currently this is used just to indicate to
+	 * iso-io whether this source is running or not. */
+}
+
+static int do_start_iso_io(struct spa_loop *loop, bool async, uint32_t seq,
+		const void *data, size_t size, void *user_data)
+{
+	struct impl *this = user_data;
+
+	spa_bt_iso_io_set_cb(this->transport->iso_io, media_iso_pull, this);
+	return 0;
+}
+
 static int transport_start(struct impl *this)
 {
 	int res, val;
@@ -669,6 +714,9 @@ static int transport_start(struct impl *this)
 				port->current_format.info.raw.rate * 80 / 1000);
 	}
 
+	this->sample_count = 0;
+	this->errqueue_count = 0;
+
 	this->source.data = this;
 
 	this->source.fd = this->fd;
@@ -679,7 +727,8 @@ static int transport_start(struct impl *this)
 		spa_log_error(this->log, "%p: failed to add poll source: %s", this,
 				spa_strerror(res));
 
-	this->sample_count = 0;
+	if (this->transport->iso_io)
+		spa_loop_invoke(this->data_loop, do_start_iso_io, 0, NULL, 0, true, this);
 
 	this->transport_started = true;
 
@@ -737,6 +786,8 @@ static int do_remove_source(struct spa_loop *loop,
 
 	if (this->timer_source.loop)
 		spa_loop_remove_source(this->data_loop, &this->timer_source);
+	if (this->transport && this->transport->iso_io)
+		spa_bt_iso_io_set_cb(this->transport->iso_io, NULL, NULL);
 	set_timeout(this, 0);
 
 	return 0;
@@ -757,6 +808,8 @@ static int do_remove_transport_source(struct spa_loop *loop,
 
 	if (this->source.loop)
 		spa_loop_remove_source(this->data_loop, &this->source);
+	if (this->transport->iso_io)
+		spa_bt_iso_io_set_cb(this->transport->iso_io, NULL, NULL);
 
 	return 0;
 }
@@ -1137,8 +1190,6 @@ static int port_set_format(struct impl *this, struct port *port,
 
 	port->info.change_mask |= SPA_PORT_CHANGE_MASK_PARAMS;
 	if (port->have_format) {
-		port->info.change_mask |= SPA_PORT_CHANGE_MASK_FLAGS;
-		port->info.flags = SPA_PORT_FLAG_LIVE;
 		port->info.change_mask |= SPA_PORT_CHANGE_MASK_RATE;
 		port->info.rate = SPA_FRACTION(1, port->current_format.info.raw.rate);
 		port->params[IDX_Format] = SPA_PARAM_INFO(SPA_PARAM_Format, SPA_PARAM_INFO_READWRITE);
@@ -1669,6 +1720,7 @@ impl_init(const struct spa_handle_factory *factory,
 	port->info = SPA_PORT_INFO_INIT();
 	port->info.change_mask = SPA_PORT_CHANGE_MASK_FLAGS;
 	port->info.flags = SPA_PORT_FLAG_LIVE |
+			   SPA_PORT_FLAG_PHYSICAL |
 			   SPA_PORT_FLAG_TERMINAL;
 	port->params[IDX_EnumFormat] = SPA_PARAM_INFO(SPA_PARAM_EnumFormat, SPA_PARAM_INFO_READ);
 	port->params[IDX_Meta] = SPA_PARAM_INFO(SPA_PARAM_Meta, SPA_PARAM_INFO_READ);
