@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2018 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2018 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 /*
  [title]
@@ -43,7 +23,7 @@
 
 #include <pipewire/pipewire.h>
 
-#define M_PI_M2 ( M_PI + M_PI )
+#define M_PI_M2f (float)(M_PI + M_PI)
 
 #define BUFFER_SAMPLES	128
 #define MAX_BUFFERS	32
@@ -84,8 +64,8 @@ struct data {
 	uint32_t n_buffers;
 	struct spa_list empty;
 
-	double accumulator;
-	double volume_accum;
+	float accumulator;
+	float volume_accum;
 };
 
 static void update_volume(struct data *data)
@@ -101,13 +81,13 @@ static void update_volume(struct data *data)
 	spa_pod_builder_control(&b, 0, SPA_CONTROL_Properties);
 	spa_pod_builder_push_object(&b, &f[1], SPA_TYPE_OBJECT_Props, 0);
 	spa_pod_builder_prop(&b, SPA_PROP_volume, 0);
-	spa_pod_builder_float(&b, (sin(data->volume_accum) / 2.0) + 0.5);
+	spa_pod_builder_float(&b, (sinf(data->volume_accum) / 2.0f) + 0.5f);
 	spa_pod_builder_pop(&b, &f[1]);
 	spa_pod_builder_pop(&b, &f[0]);
 
-        data->volume_accum += M_PI_M2 / 1000.0;
-        if (data->volume_accum >= M_PI_M2)
-                data->volume_accum -= M_PI_M2;
+        data->volume_accum += M_PI_M2f / 1000.0f;
+        if (data->volume_accum >= M_PI_M2f)
+                data->volume_accum -= M_PI_M2f;
 }
 
 static int impl_send_command(void *object, const struct spa_command *command)
@@ -339,14 +319,13 @@ static int impl_port_use_buffers(void *object,
 		}
 		else if (datas[0].type == SPA_DATA_MemFd ||
 			 datas[0].type == SPA_DATA_DmaBuf) {
-			b->ptr = mmap(NULL, datas[0].maxsize + datas[0].mapoffset, PROT_WRITE,
-				      MAP_SHARED, datas[0].fd, 0);
+			b->ptr = mmap(NULL, datas[0].maxsize, PROT_WRITE,
+				      MAP_SHARED, datas[0].fd, datas[0].mapoffset);
 			if (b->ptr == MAP_FAILED) {
 				pw_log_error("failed to buffer mem");
 				return -errno;
 
 			}
-			b->ptr = SPA_PTROFF(b->ptr, datas[0].mapoffset, void);
 			b->mapped = true;
 		}
 		else {
@@ -385,11 +364,11 @@ static void fill_f32(struct data *d, void *dest, int avail)
         for (i = 0; i < n_samples; i++) {
 		float val;
 
-                d->accumulator += M_PI_M2 * 440 / d->format.rate;
-                if (d->accumulator >= M_PI_M2)
-                        d->accumulator -= M_PI_M2;
+                d->accumulator += M_PI_M2f * 440 / d->format.rate;
+                if (d->accumulator >= M_PI_M2f)
+                        d->accumulator -= M_PI_M2f;
 
-                val = sin(d->accumulator);
+                val = sinf(d->accumulator);
 
                 for (c = 0; c < d->format.channels; c++)
                         *dst++ = val;
@@ -406,11 +385,11 @@ static void fill_s16(struct data *d, void *dest, int avail)
         for (i = 0; i < n_samples; i++) {
                 int16_t val;
 
-                d->accumulator += M_PI_M2 * 440 / d->format.rate;
-                if (d->accumulator >= M_PI_M2)
-                        d->accumulator -= M_PI_M2;
+                d->accumulator += M_PI_M2f * 440 / d->format.rate;
+                if (d->accumulator >= M_PI_M2f)
+                        d->accumulator -= M_PI_M2f;
 
-                val = (int16_t) (sin(d->accumulator) * 32767.0);
+                val = (int16_t) (sinf(d->accumulator) * 32767.0f);
 
                 for (c = 0; c < d->format.channels; c++)
                         *dst++ = val;
