@@ -923,7 +923,7 @@ handle_dmabuf_prop (const struct spa_pod_prop *prop,
         g_ptr_array_add(fmt_array, g_strdup_printf ("%s", fmt_str));
 
 #ifdef HAVE_GSTREAMER_DMA_DRM
-      {
+      if (mods[j] != DRM_FORMAT_MOD_INVALID) {
         char *drm_str;
 
         if ((drm_str = video_id_to_dma_drm_fourcc(id[i], mods[j])))
