@@ -1,26 +1,6 @@
-/* Spa
- *
- * Copyright © 2022 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* Spa */
+/* SPDX-FileCopyrightText: Copyright © 2022 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #include <string.h>
 #include <stdio.h>
@@ -47,6 +27,8 @@ static struct dsp_info dsp_table[] =
 		.funcs.mix_gain = dsp_mix_gain_sse,
 		.funcs.biquad_run = dsp_biquad_run_c,
 		.funcs.sum = dsp_sum_avx,
+		.funcs.linear = dsp_linear_c,
+		.funcs.mult = dsp_mult_c,
 		.funcs.fft_new = dsp_fft_new_c,
 		.funcs.fft_free = dsp_fft_free_c,
 		.funcs.fft_run = dsp_fft_run_c,
@@ -61,6 +43,8 @@ static struct dsp_info dsp_table[] =
 		.funcs.mix_gain = dsp_mix_gain_sse,
 		.funcs.biquad_run = dsp_biquad_run_c,
 		.funcs.sum = dsp_sum_sse,
+		.funcs.linear = dsp_linear_c,
+		.funcs.mult = dsp_mult_c,
 		.funcs.fft_new = dsp_fft_new_c,
 		.funcs.fft_free = dsp_fft_free_c,
 		.funcs.fft_run = dsp_fft_run_c,
@@ -74,6 +58,8 @@ static struct dsp_info dsp_table[] =
 		.funcs.mix_gain = dsp_mix_gain_c,
 		.funcs.biquad_run = dsp_biquad_run_c,
 		.funcs.sum = dsp_sum_c,
+		.funcs.linear = dsp_linear_c,
+		.funcs.mult = dsp_mult_c,
 		.funcs.fft_new = dsp_fft_new_c,
 		.funcs.fft_free = dsp_fft_free_c,
 		.funcs.fft_run = dsp_fft_run_c,
@@ -98,14 +84,15 @@ static void impl_dsp_ops_free(struct dsp_ops *ops)
 	spa_zero(*ops);
 }
 
-int dsp_ops_init(struct dsp_ops *ops)
+int dsp_ops_init(struct dsp_ops *ops, uint32_t cpu_flags)
 {
 	const struct dsp_info *info;
 
-	info = find_dsp_info(ops->cpu_flags);
+	info = find_dsp_info(cpu_flags);
 	if (info == NULL)
 		return -ENOTSUP;
 
+	ops->cpu_flags = cpu_flags;
 	ops->priv = info;
 	ops->free = impl_dsp_ops_free;
 	ops->funcs = info->funcs;
