@@ -85,6 +85,8 @@
  * ## Example configuration of a duplex sink/source
  *
  *\code{.unparsed}
+ * # ~/.config/pipewire/pipewire.conf.d/my-netjack2-driver.conf
+ *
  * context.modules = [
  * {   name = libpipewire-module-netjack2-driver
  *     args = {
@@ -274,6 +276,8 @@ static void stream_state_changed(void *d, enum pw_filter_state old,
 	struct impl *impl = s->impl;
 	switch (state) {
 	case PW_FILTER_STATE_ERROR:
+		pw_log_warn("stream %p: error: %s", s, error);
+		break;
 	case PW_FILTER_STATE_UNCONNECTED:
 		pw_impl_module_schedule_destroy(impl->module);
 		break;
