@@ -1,26 +1,6 @@
-/* PipeWire
- *
- * Copyright © 2020 Wim Taymans
- *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice (including the next
- * paragraph) shall be included in all copies or substantial portions of the
- * Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
- */
+/* PipeWire */
+/* SPDX-FileCopyrightText: Copyright © 2020 Wim Taymans */
+/* SPDX-License-Identifier: MIT */
 
 #ifndef PULSER_SERVER_CLIENT_H
 #define PULSER_SERVER_CLIENT_H
@@ -94,8 +74,6 @@ struct client {
 
 	struct spa_list pending_samples;
 
-	struct spa_list pending_streams;
-
 	unsigned int disconnect:1;
 	unsigned int new_msg_since_last_flush:1;
 	unsigned int authenticated:1;
@@ -107,10 +85,12 @@ struct client {
 };
 
 struct client_events {
-#define VERSION_CLIENT_EVENTS	0
+#define VERSION_CLIENT_EVENTS	1
 	uint32_t version;
 
 	void (*disconnect) (void *data);
+
+	void (*routes_changed) (void *data);
 };
 
 struct client *client_new(struct server *server);
@@ -121,6 +101,8 @@ int client_queue_message(struct client *client, struct message *msg);
 int client_flush_messages(struct client *client);
 int client_queue_subscribe_event(struct client *client, uint32_t mask, uint32_t event, uint32_t id);
 
+void client_update_routes(struct client *client, const char *key, const char *value);
+
 static inline void client_unref(struct client *client)
 {
 	if (--client->ref == 0)
